@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 5027 2008-02-13 16:31:34Z charles $
+ * $Id: main.c 5314 2008-03-19 20:34:35Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,7 +23,6 @@
  *****************************************************************************/
 
 #include <sys/param.h>
-#include <errno.h>
 #include <signal.h>
 #include <string.h>
 #include <stdio.h>
@@ -43,17 +42,20 @@
 
 #include "actions.h"
 #include "conf.h"
+#include "details.h"
 #include "dialogs.h"
+#include "hig.h"
 #include "ipc.h"
 #include "makemeta-ui.h"
 #include "msgwin.h"
+#include "notify.h"
+#include "open-dialog.h"
 #include "stats.h"
-#include "torrent-inspector.h"
-#include "tr_core.h"
-#include "tr_icon.h"
-#include "tr_prefs.h"
-#include "tr_torrent.h"
-#include "tr_window.h"
+#include "tr-core.h"
+#include "tr-icon.h"
+#include "tr-prefs.h"
+#include "tr-torrent.h"
+#include "tr-window.h"
 #include "util.h"
 #include "ui.h"
 
@@ -61,14 +63,11 @@
 #include <libtransmission/version.h>
 
 /* interval in milliseconds to update the torrent list display */
-#define UPDATE_INTERVAL         1000
+#define UPDATE_INTERVAL         1666
 
 /* interval in milliseconds to check for stopped torrents and update display */
 #define EXIT_CHECK_INTERVAL     500
 
-/* number of fatal signals required to cause an immediate exit */
-#define SIGCOUNT_MAX            3
-
 #if GTK_CHECK_VERSION(2,8,0)
 #define SHOW_LICENSE
 static const char * LICENSE = 
@@ -104,37 +103,33 @@ static const char * LICENSE =
 
 struct cbdata
 {
+    gboolean       minimized;
+    gboolean       closing;
+    guint          timer;
     guint          idle_hide_mainwindow_tag;
+    gpointer       icon;
     GtkWindow    * wind;
     TrCore       * core;
-    gpointer       icon;
     GtkWidget    * msgwin;
     GtkWidget    * prefs;
-    guint          timer;
-    gboolean       closing;
-    GList        * errqueue;
+    GSList       * errqueue;
     GHashTable   * tor2details;
     GHashTable   * details2tor;
-    gboolean       minimized;
 };
 
 #define CBDATA_PTR "callback-data-pointer"
 
 static GtkUIManager * myUIManager = NULL;
 
-static sig_atomic_t global_sigcount = 0;
-
 static gboolean
-sendremote( GList * files, gboolean sendquit );
+sendremote( GSList * files, gboolean sendquit );
 static void
-appsetup( TrWindow * wind, GList * args,
+appsetup( TrWindow * wind, GSList * args,
           struct cbdata *,
           gboolean paused, gboolean minimized );
 static void
 winsetup( struct cbdata * cbdata, TrWindow * wind );
 static void
-makeicon( struct cbdata * cbdata );
-static void
 wannaquit( void * vdata );
 static void
 setupdrag(GtkWidget *widget, struct cbdata *data);
@@ -146,22 +141,11 @@ static void
 coreerr( TrCore * core, enum tr_core_err code, const char * msg,
          gpointer gdata );
 static void
-coreprompt( TrCore *, GList *, enum tr_torrent_action, gboolean, gpointer );
-static void
-corepromptdata( TrCore *, uint8_t *, size_t, gboolean, gpointer );
-static void
-initializeFromPrefs( struct cbdata * cbdata );
+onAddTorrent( TrCore *, tr_ctor *, gpointer );
 static void
 prefschanged( TrCore * core, const char * key, gpointer data );
 static gboolean
 updatemodel(gpointer gdata);
-static GList *
-getselection( struct cbdata * cbdata );
-
-static void
-setupsighandlers(void);
-static void
-fatalsig(int sig);
 
 struct counts_data
 {
@@ -213,12 +197,21 @@ refreshTorrentActions( GtkTreeSelection * s )
     action_sensitize( "pause-torrent", counts.activeCount!=0 );
     action_sensitize( "start-torrent", counts.inactiveCount!=0 );
     action_sensitize( "remove-torrent", counts.totalCount!=0 );
+    action_sensitize( "delete-torrent", counts.totalCount!=0 );
     action_sensitize( "verify-torrent", counts.totalCount!=0 );
     action_sensitize( "show-torrent-details", counts.totalCount==1 );
 
     canUpdate = 0;
     gtk_tree_selection_selected_foreach( s, accumulateCanUpdateForeach, &canUpdate );
     action_sensitize( "update-tracker", canUpdate!=0 );
+
+    {
+        GtkTreeView * view = gtk_tree_selection_get_tree_view( s );
+        GtkTreeModel * model = gtk_tree_view_get_model( view );
+        const int torrentCount = gtk_tree_model_iter_n_children( model, NULL ) != 0;
+        action_sensitize( "select-all", torrentCount!=0 );
+        action_sensitize( "deselect-all", torrentCount!=0 );
+    }
 }
 
 static void
@@ -227,12 +220,51 @@ selectionChangedCB( GtkTreeSelection * s, gpointer unused UNUSED )
     refreshTorrentActions( s );
 }
 
+static void
+windowStateChanged( GtkWidget * widget UNUSED, GdkEventWindowState * event, gpointer gdata )
+{
+    if( event->changed_mask & GDK_WINDOW_STATE_ICONIFIED )
+    {
+        struct cbdata * cbdata = gdata;
+        cbdata->minimized = ( event->new_window_state & GDK_WINDOW_STATE_ICONIFIED ) ? 1 : 0;
+    }
+}
+
+static sig_atomic_t global_sigcount = 0;
+
+static void
+fatalsig( int sig )
+{
+    static const int SIGCOUNT_MAX = 3; /* revert to default handler after this many */
+
+    if( ++global_sigcount >= SIGCOUNT_MAX )
+    {
+        signal( sig, SIG_DFL );
+        raise( sig );
+    }
+}
+
+
+static void
+setupsighandlers( void )
+{
+#ifdef G_OS_WIN32
+  const int sigs[] = { SIGINT, SIGTERM };
+#else
+  const int sigs[] = { SIGHUP, SIGINT, SIGQUIT, SIGTERM };
+#endif
+  guint i;
+
+  for( i=0; i<G_N_ELEMENTS(sigs); ++i )
+      signal( sigs[i], fatalsig );
+}
+
 int
 main( int argc, char ** argv )
 {
     char * err;
     struct cbdata * cbdata;
-    GList * argfiles;
+    GSList * argfiles;
     GError * gerr;
     gboolean didinit = FALSE;
     gboolean didlock = FALSE;
@@ -244,7 +276,7 @@ main( int argc, char ** argv )
         { "paused", 'p', 0, G_OPTION_ARG_NONE, &startpaused,
           _("Start with all torrents paused"), NULL },
         { "quit", 'q', 0, G_OPTION_ARG_NONE, &sendquit,
-          _( "Request that the running instance quit"), NULL },
+          _( "Ask the running instance to quit"), NULL },
 #ifdef STATUS_ICON_SUPPORTED
         { "minimized", 'm', 0, G_OPTION_ARG_NONE, &startminimized,
           _( "Start minimized in system tray"), NULL },
@@ -271,6 +303,8 @@ main( int argc, char ** argv )
         return 0;
     }
 
+    tr_notify_init( );
+
     didinit = cf_init( tr_getPrefsDirectory(), NULL ); /* must come before actions_init */
     tr_prefs_init_global( );
     myUIManager = gtk_ui_manager_new ();
@@ -290,9 +324,7 @@ main( int argc, char ** argv )
 
         /* create main window now to be a parent to any error dialogs */
         GtkWindow * mainwind = GTK_WINDOW( tr_window_new( myUIManager, cbdata->core ) );
-
-        /* set message level here before tr_init() */
-        msgwin_loadpref( );
+        g_signal_connect( mainwind, "window-state-event", G_CALLBACK(windowStateChanged), cbdata );
 
         appsetup( mainwind, argfiles, cbdata, startpaused, startminimized );
     }
@@ -311,7 +343,7 @@ main( int argc, char ** argv )
 }
 
 static gboolean
-sendremote( GList * files, gboolean sendquit )
+sendremote( GSList * files, gboolean sendquit )
 {
     const gboolean didlock = cf_lock( NULL );
 
@@ -327,11 +359,12 @@ sendremote( GList * files, gboolean sendquit )
 }
 
 static void
-appsetup( TrWindow * wind, GList * args,
+appsetup( TrWindow * wind, GSList * torrentFiles,
           struct cbdata * cbdata,
-          gboolean paused, gboolean minimized )
+          gboolean forcepause, gboolean minimized )
 {
-    enum tr_torrent_action action;
+    const pref_flag_t start = forcepause ? PREF_FLAG_FALSE : PREF_FLAG_DEFAULT;
+    const pref_flag_t prompt = PREF_FLAG_DEFAULT;
 
     /* fill out cbdata */
     cbdata->wind       = NULL;
@@ -347,29 +380,17 @@ appsetup( TrWindow * wind, GList * args,
 
     /* set up core handlers */
     g_signal_connect( cbdata->core, "error", G_CALLBACK( coreerr ), cbdata );
-    g_signal_connect( cbdata->core, "directory-prompt",
-                      G_CALLBACK( coreprompt ), cbdata );
-    g_signal_connect( cbdata->core, "directory-prompt-data",
-                      G_CALLBACK( corepromptdata ), cbdata );
+    g_signal_connect( cbdata->core, "add-torrent-prompt",
+                      G_CALLBACK( onAddTorrent ), cbdata );
     g_signal_connect_swapped( cbdata->core, "quit",
                               G_CALLBACK( wannaquit ), cbdata );
     g_signal_connect( cbdata->core, "prefs-changed",
                       G_CALLBACK( prefschanged ), cbdata );
 
-    /* apply a few prefs */
-
-    if( minimized )
-        tr_core_set_pref_bool( cbdata->core, PREF_KEY_SYSTRAY, TRUE );
-    initializeFromPrefs( cbdata );
-
     /* add torrents from command-line and saved state */
-    tr_core_load( cbdata->core, paused );
-
-    if( NULL != args )
-    {
-        action = tr_prefs_get_action( PREF_KEY_ADDIPC );
-        tr_core_add_list( cbdata->core, args, action, paused );
-    }
+    tr_core_load( cbdata->core, forcepause );
+    tr_core_add_list( cbdata->core, torrentFiles, start, prompt );
+    torrentFiles = NULL;
     tr_core_torrents_added( cbdata->core );
 
     /* set up the ipc socket */
@@ -378,6 +399,9 @@ appsetup( TrWindow * wind, GList * args,
     /* set up main window */
     winsetup( cbdata, wind );
 
+    /* set up the system tray */
+    cbdata->icon = tr_icon_new( cbdata->core );
+
     /* start model update timer */
     cbdata->timer = g_timeout_add( UPDATE_INTERVAL, updatemodel, cbdata );
     updatemodel( cbdata );
@@ -443,7 +467,7 @@ static void
 toggleMainWindow( struct cbdata * cbdata )
 {
     GtkWindow * window = GTK_WINDOW( cbdata->wind );
-    const int hide = cbdata->minimized = !cbdata->minimized;
+    const int hide = !cbdata->minimized;
     static int x=0, y=0;
 
     if( hide )
@@ -510,13 +534,6 @@ winsetup( struct cbdata * cbdata, TrWindow * wind )
     setupdrag( GTK_WIDGET(wind), cbdata );
 }
 
-static void
-makeicon( struct cbdata * cbdata )
-{
-    if( cbdata->icon == NULL )
-        cbdata->icon = tr_icon_new( );
-}
-
 static gpointer
 quitThreadFunc( gpointer gdata )
 {
@@ -533,8 +550,8 @@ quitThreadFunc( gpointer gdata )
     if( cbdata->icon )
         g_object_unref( cbdata->icon );
     if( cbdata->errqueue ) {
-        g_list_foreach( cbdata->errqueue, (GFunc)g_free, NULL );
-        g_list_free( cbdata->errqueue );
+        g_slist_foreach( cbdata->errqueue, (GFunc)g_free, NULL );
+        g_slist_free( cbdata->errqueue );
     }
 
     g_hash_table_destroy( cbdata->details2tor );
@@ -567,7 +584,6 @@ do_exit_cb( GtkWidget *w UNUSED, gpointer data UNUSED )
 static void
 wannaquit( void * vdata )
 {
-    char * str;
     GtkWidget * r, * p, * b, * w, *c;
     struct cbdata * cbdata = vdata;
 
@@ -583,24 +599,28 @@ wannaquit( void * vdata )
     r = gtk_alignment_new(0.5, 0.5, 0.01, 0.01);
     gtk_container_add(GTK_CONTAINER(c), r);
 
-    p = gtk_table_new(2, 2, FALSE);
-    gtk_container_add(GTK_CONTAINER(r), p);
+    p = gtk_table_new(3, 2, FALSE);
+    gtk_table_set_col_spacings( GTK_TABLE( p ), GUI_PAD_BIG );
+    gtk_container_add( GTK_CONTAINER( r ), p );
 
     w = gtk_image_new_from_stock( GTK_STOCK_NETWORK, GTK_ICON_SIZE_DIALOG );
-    gtk_table_attach(GTK_TABLE(p), w, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
+    gtk_table_attach_defaults(GTK_TABLE(p), w, 0, 1, 0, 2 );
 
-    w = gtk_label_new("");
-    str = g_strdup_printf( "<b>%s</b>\n%s", _("Closing Connections"), _("Sending upload/download totals to tracker...") );
-    gtk_label_set_markup(GTK_LABEL(w), str );
-    gtk_table_attach(GTK_TABLE(p), w, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 10, 0);
-    g_free( str );
+    w = gtk_label_new( NULL );
+    gtk_label_set_markup( GTK_LABEL( w ), _( "<b>Closing Connections</b>" ) );
+    gtk_misc_set_alignment( GTK_MISC( w ), 0.0, 0.5 );
+    gtk_table_attach_defaults( GTK_TABLE( p ), w, 1, 2, 0, 1 );
+
+    w = gtk_label_new( _( "Sending upload/download totals to tracker..." ) );
+    gtk_misc_set_alignment( GTK_MISC( w ), 0.0, 0.5 );
+    gtk_table_attach_defaults( GTK_TABLE( p ), w, 1, 2, 1, 2 );
 
     b = gtk_alignment_new(0.0, 1.0, 0.01, 0.01);
     w = gtk_button_new_with_label( _( "_Quit Immediately" ) );
     gtk_button_set_image( GTK_BUTTON(w), gtk_image_new_from_stock( GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON ) );
     g_signal_connect(w, "clicked", G_CALLBACK(do_exit_cb), NULL);
     gtk_container_add(GTK_CONTAINER(b), w);
-    gtk_table_attach(GTK_TABLE(p), b, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 10, 0);
+    gtk_table_attach(GTK_TABLE(p), b, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 10 );
 
     gtk_widget_show_all(r);
 
@@ -622,10 +642,11 @@ gotdrag( GtkWidget         * widget UNUSED,
          gpointer            gdata )
 {
     struct cbdata * data = gdata;
-    GList * paths = NULL;
-    GList * freeme = NULL;
+    GSList * paths = NULL;
+    GSList * freeme = NULL;
 
-#ifdef DND_DEBUG
+#if 0
+    int i;
     char *sele = gdk_atom_name(sel->selection);
     char *targ = gdk_atom_name(sel->target);
     char *type = gdk_atom_name(sel->type);
@@ -655,8 +676,8 @@ gotdrag( GtkWidget         * widget UNUSED,
                 continue;
 
             /* decode the filename */
-            filename = urldecode( files[i], -1 );
-            freeme = g_list_prepend( freeme, filename );
+            filename = decode_uri( files[i] );
+            freeme = g_slist_prepend( freeme, filename );
             if( !g_utf8_validate( filename, -1, NULL ) )
                 continue;
 
@@ -677,17 +698,15 @@ gotdrag( GtkWidget         * widget UNUSED,
 
             /* finally, add it to the list of torrents to try adding */
             if( g_file_test( filename, G_FILE_TEST_EXISTS ) )
-                paths = g_list_prepend( paths, filename );
+                paths = g_slist_prepend( paths, g_strdup( filename ) );
         }
 
         /* try to add any torrents we found */
-        if( paths != NULL )
+        if( paths )
         {
-            enum tr_torrent_action action = tr_prefs_get_action( PREF_KEY_ADDSTD );
-            paths = g_list_reverse( paths );
-            tr_core_add_list( data->core, paths, action, FALSE );
+            paths = g_slist_reverse( paths );
+            tr_core_add_list_defaults( data->core, paths );
             tr_core_torrents_added( data->core );
-            g_list_free( paths );
         }
 
         freestrlist( freeme );
@@ -722,20 +741,20 @@ coreerr( TrCore * core UNUSED, enum tr_core_err code, const char * msg,
     switch( code )
     {
         case TR_CORE_ERR_ADD_TORRENT:
-            cbdata->errqueue = g_list_append( cbdata->errqueue,
-                                              g_strdup( msg ) );
+            cbdata->errqueue = g_slist_append( cbdata->errqueue,
+                                               g_strdup( msg ) );
             return;
         case TR_CORE_ERR_NO_MORE_TORRENTS:
-            if( NULL != cbdata->errqueue )
+            if( cbdata->errqueue )
             {
                 joined = joinstrlist( cbdata->errqueue, "\n" );
                 errmsg( cbdata->wind,
-                        ngettext( "Failed to load torrent file:\n%s",
-                                  "Failed to load torrent files:\n%s",
-                                  g_list_length( cbdata->errqueue ) ),
+                        ngettext( "Failed to load torrent file: %s",
+                                  "Failed to load torrent files: %s",
+                                  g_slist_length( cbdata->errqueue ) ),
                         joined );
-                g_list_foreach( cbdata->errqueue, (GFunc) g_free, NULL );
-                g_list_free( cbdata->errqueue );
+                g_slist_foreach( cbdata->errqueue, (GFunc) g_free, NULL );
+                g_slist_free( cbdata->errqueue );
                 cbdata->errqueue = NULL;
                 g_free( joined );
             }
@@ -748,28 +767,26 @@ coreerr( TrCore * core UNUSED, enum tr_core_err code, const char * msg,
     g_assert_not_reached();
 }
 
-void
-coreprompt( TrCore * core, GList * paths, enum tr_torrent_action act,
-            gboolean paused, gpointer gdata )
+#if GTK_CHECK_VERSION(2,8,0)
+static void
+on_main_window_focus_in( GtkWidget      * widget UNUSED,
+                         GdkEventFocus  * event UNUSED,
+                         gpointer         gdata )
 {
     struct cbdata * cbdata = gdata;
-
-    promptfordir( cbdata->wind, core, paths, NULL, 0, act, paused );
-}
-
-void
-corepromptdata( TrCore * core, uint8_t * data, size_t size,
-                gboolean paused, gpointer gdata )
-{
-    struct cbdata * cbdata = gdata;
-
-    promptfordir( cbdata->wind, core, NULL, data, size, TR_TOR_LEAVE, paused );
+    gtk_window_set_urgency_hint( GTK_WINDOW( cbdata->wind ), FALSE );
 }
+#endif
 
 static void
-initializeFromPrefs( struct cbdata * cbdata )
+onAddTorrent( TrCore * core, tr_ctor * ctor, gpointer gdata )
 {
-    prefschanged( NULL, PREF_KEY_SYSTRAY, cbdata );
+    struct cbdata * cbdata = gdata;
+    GtkWidget * w = openSingleTorrentDialog( cbdata->wind, core, ctor );
+#if GTK_CHECK_VERSION(2,8,0)
+    g_signal_connect( w, "focus-in-event", G_CALLBACK(on_main_window_focus_in),  cbdata );
+    gtk_window_set_urgency_hint( cbdata->wind, TRUE );
+#endif
 }
 
 static void
@@ -814,18 +831,6 @@ prefschanged( TrCore * core UNUSED, const char * key, gpointer data )
         const gboolean enabled = pref_flag_get( key );
         tr_natTraversalEnable( tr, enabled );
     }
-    else if( !strcmp( key, PREF_KEY_SYSTRAY ) )
-    {
-        if( pref_flag_get( key ) )
-        {
-            makeicon( cbdata );
-        }
-        else if( cbdata->icon )
-        {
-            g_object_unref( cbdata->icon );
-            cbdata->icon = NULL;
-        }
-    }
     else if( !strcmp( key, PREF_KEY_PEX ) )
     {
         const gboolean enabled = pref_flag_get( key );
@@ -850,41 +855,9 @@ updatemodel(gpointer gdata) {
   if( data->wind )
       tr_window_update( data->wind );
 
-  /* update the message window */
-  msgwin_update();
-
   return TRUE;
 }
 
-/* returns a GList of GtkTreeRowReferences to each selected row */
-static GList *
-getselection( struct cbdata * cbdata )
-{
-    GList * rows = NULL;
-
-    if( NULL != cbdata->wind )
-    {
-        GList * l;
-        GtkTreeSelection *s = tr_window_get_selection(cbdata->wind);
-        GtkTreeModel * filter_model;
-        GtkTreeModel * store_model;
-        rows = gtk_tree_selection_get_selected_rows( s, &filter_model );
-        store_model = gtk_tree_model_filter_get_model(
-                                                 GTK_TREE_MODEL_FILTER( filter_model ) );
-        for( l=rows; l!=NULL; l=l->next )
-        {
-            GtkTreePath * path = gtk_tree_model_filter_convert_path_to_child_path(
-                                        GTK_TREE_MODEL_FILTER( filter_model ), l->data );
-            GtkTreeRowReference * ref = gtk_tree_row_reference_new( store_model, path );
-            gtk_tree_path_free( path );
-            gtk_tree_path_free( l->data );
-            l->data = ref;
-        }
-    }
-
-    return rows;
-}
-
 static void
 about ( GtkWindow * parent )
 {
@@ -910,6 +883,8 @@ about ( GtkWindow * parent )
         "wrap-license", TRUE,
 #endif
         "authors", authors,
+        /* Translators: translate "translator-credits" as your name
+           to have it appear in the credits in the "About" dialog */
         "translator-credits", _("translator-credits"),
         NULL );
 }
@@ -994,26 +969,51 @@ recheckTorrentForeach (GtkTreeModel * model,
 {
     TrTorrent * gtor = NULL;
     gtk_tree_model_get( model, iter, MC_TORRENT, &gtor, -1 );
-    tr_torrentRecheck( tr_torrent_handle( gtor ) );
+    tr_torrentVerify( tr_torrent_handle( gtor ) );
     g_object_unref( G_OBJECT( gtor ) );
 }
 
 static gboolean 
-msgwinclosed()
+msgwinclosed( void )
 {
   action_toggle( "toggle-message-log", FALSE );
   return FALSE;
 }
 
+static void
+accumulateSelectedTorrents( GtkTreeModel * model,
+                            GtkTreePath  * path UNUSED,
+                            GtkTreeIter  * iter,
+                            gpointer       gdata )
+{
+    GSList ** data = ( GSList** ) gdata;
+    TrTorrent * tor = NULL;
+    gtk_tree_model_get( model, iter, MC_TORRENT, &tor, -1 );
+    *data = g_slist_prepend( *data, tor );
+}
+
+static void
+removeSelected( struct cbdata * data, gboolean delete_files )
+{
+    GSList * l = NULL;
+    GtkTreeSelection * s = tr_window_get_selection( data->wind );
+    gtk_tree_selection_selected_foreach( s, accumulateSelectedTorrents, &l );
+    gtk_tree_selection_unselect_all( s );
+    if( l ) {
+        l = g_slist_reverse( l );
+        confirmRemove( data->wind, data->core, l, delete_files );
+    }
+}
+
 void
 doAction ( const char * action_name, gpointer user_data )
 {
     struct cbdata * data = user_data;
     gboolean changed = FALSE;
 
-    if (!strcmp (action_name, "add-torrent"))
+    if ( !strcmp (action_name, "open-torrent-menu") || !strcmp( action_name, "open-torrent-toolbar" ))
     {
-        makeaddwind( data->wind, data->core );
+        openDialog( data->wind, data->core );
     }
     else if (!strcmp (action_name, "show-stats"))
     {
@@ -1049,30 +1049,19 @@ doAction ( const char * action_name, gpointer user_data )
         GtkTreeSelection * s = tr_window_get_selection(data->wind);
         gtk_tree_selection_selected_foreach( s, updateTrackerForeach, data->wind );
     }
-    else if (!strcmp (action_name, "create-torrent"))
+    else if (!strcmp (action_name, "new-torrent"))
     {
         GtkWidget * w = make_meta_ui( GTK_WINDOW( data->wind ),
                                       tr_core_handle( data->core ) );
         gtk_widget_show_all( w );
     }
-    else if (!strcmp (action_name, "remove-torrent"))
+    else if( !strcmp( action_name, "remove-torrent" ) )
     {
-        /* this modifies the model's contents, so can't use foreach */
-        GList *l, *sel = getselection( data );
-        GtkTreeModel *model = tr_core_model( data->core );
-        gtk_tree_selection_unselect_all( tr_window_get_selection( data->wind) );
-        for( l=sel; l!=NULL; l=l->next )
-        {
-            GtkTreeIter iter;
-            GtkTreeRowReference * reference = l->data;
-            GtkTreePath * path = gtk_tree_row_reference_get_path( reference );
-            gtk_tree_model_get_iter( model, &iter, path );
-            tr_core_delete_torrent( data->core, &iter );
-            gtk_tree_path_free( path );
-            gtk_tree_row_reference_free( reference );
-            changed = TRUE;
-        }
-        g_list_free( sel );
+        removeSelected( data, FALSE );
+    }
+    else if( !strcmp( action_name, "delete-torrent" ) )
+    {
+        removeSelected( data, TRUE );
     }
     else if (!strcmp (action_name, "close"))
     {
@@ -1088,7 +1077,7 @@ doAction ( const char * action_name, gpointer user_data )
         GtkTreeSelection * s = tr_window_get_selection(data->wind);
         gtk_tree_selection_select_all( s );
     }
-    else if (!strcmp (action_name, "unselect-all"))
+    else if (!strcmp (action_name, "deselect-all"))
     {
         GtkTreeSelection * s = tr_window_get_selection(data->wind);
         gtk_tree_selection_unselect_all( s );
@@ -1107,7 +1096,7 @@ doAction ( const char * action_name, gpointer user_data )
     {
         if( !data->msgwin )
         {
-            GtkWidget * win = msgwin_create( data->core );
+            GtkWidget * win = msgwin_new( data->core );
             g_signal_connect( win, "destroy", G_CALLBACK( msgwinclosed ), 
                              NULL );
             data->msgwin = win;
@@ -1132,28 +1121,3 @@ doAction ( const char * action_name, gpointer user_data )
     if( changed )
         updatemodel( data );
 }
-
-
-static void
-setupsighandlers(void) {
-  int sigs[] = {SIGHUP, SIGINT, SIGQUIT, SIGTERM};
-  struct sigaction sa;
-  int ii;
-
-  memset(&sa, 0,  sizeof(sa));
-  sa.sa_handler = fatalsig;
-  for(ii = 0; ii < ALEN(sigs); ii++)
-    sigaction(sigs[ii], &sa, NULL);
-}
-
-static void
-fatalsig(int sig) {
-  struct sigaction sa;
-
-  if(SIGCOUNT_MAX <= ++global_sigcount) {
-    memset(&sa, 0,  sizeof(sa));
-    sa.sa_handler = SIG_DFL;
-    sigaction(sig, &sa, NULL);
-    raise(sig);
-  }
-}
