@@ -7,65 +7,31 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id:$
+ * $Id: torrent-cell-renderer.c 5313 2008-03-19 20:07:27Z charles $
  */
 
 #include "assert.h"
 #include <gtk/gtk.h>
 #include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcellrendererprogress.h>
 #include <glib/gi18n.h>
 #include <libtransmission/transmission.h>
 #include "hig.h"
 #include "torrent-cell-renderer.h"
-#include "tr_torrent.h"
+#include "tr-torrent.h"
 #include "util.h"
 
+/* #define TEST_RTL */
+
 enum
 {
     P_TORRENT = 1,
     P_BAR_HEIGHT,
-    P_MINIMAL,
-    P_SHOW_UNAVAILABLE,
-    P_GRADIENT,
-    P_COLOR_VERIFIED,
-    P_COLOR_VERIFIED_2,
-    P_COLOR_MISSING,
-    P_COLOR_MISSING_2,
-    P_COLOR_UNWANTED,
-    P_COLOR_UNWANTED_2,
-    P_COLOR_UNAVAILABLE,
-    P_COLOR_UNAVAILABLE_2,
-    P_COLOR_PAUSED,
-    P_COLOR_PAUSED_2,
-    P_COLOR_VERIFYING,
-    P_COLOR_VERIFYING_2,
-    P_COLOR_SEEDING,
-    P_COLOR_SEEDING_2
+    P_MINIMAL
 };
 
 #define DEFAULT_BAR_HEIGHT 12
 
-#define DEFAULT_COLOR_VERIFIED       "#729fcf"
-#define DEFAULT_COLOR_VERIFIED_2     "#204a87"
-
-#define DEFAULT_COLOR_SEEDING        "#8ae234"
-#define DEFAULT_COLOR_SEEDING_2      "#4e9a06"
-
-#define DEFAULT_COLOR_MISSING        "#eeeeec" /* aluminum 1 */
-#define DEFAULT_COLOR_MISSING_2      "#babdb6" /* aluminum 3 */
-
-#define DEFAULT_COLOR_UNWANTED       "#babdb6" /* aluminum 3 */
-#define DEFAULT_COLOR_UNWANTED_2     "#2e3436" /* aluminum 6 */
-
-#define DEFAULT_COLOR_UNAVAILABLE    "#ef2929"
-#define DEFAULT_COLOR_UNAVAILABLE_2  "#a40000"
-
-#define DEFAULT_COLOR_PAUSED         "#d3d7cf" /* aluminum 2 */
-#define DEFAULT_COLOR_PAUSED_2       "#555753" /* aluminum 5 */
-
-#define DEFAULT_COLOR_VERIFYING      "#fce94f"
-#define DEFAULT_COLOR_VERIFYING_2    "#c4a000"
-
 /***
 ****
 ***/
@@ -76,29 +42,58 @@ getProgressString( const tr_info * info, const tr_stat * torStat )
     const int isDone = torStat->leftUntilDone == 0;
     const uint64_t haveTotal = torStat->haveUnchecked + torStat->haveValid;
     const int isSeed = torStat->haveValid >= info->totalSize;
-    char buf1[32], buf2[32], buf3[32];
+    char buf1[32], buf2[32], buf3[32], buf4[32];
     char * str;
 
     if( !isDone )
         str = g_strdup_printf(
-                  _("%s of %s (%.2f%%)"),
+                  /* %1$s is how much we've got,
+                     %2$s is how much we'll have when done,
+                     %3$.2f%% is a percentage of the two */
+                  _("%1$s of %2$s (%3$.2f%%)"),
                   tr_strlsize( buf1, haveTotal, sizeof(buf1) ),
                   tr_strlsize( buf2, torStat->desiredSize, sizeof(buf2) ),
                   torStat->percentDone * 100.0 );
     else if( !isSeed )
         str = g_strdup_printf(
-                  _("%s of %s (%.2f%%), uploaded %s (Ratio: %.1f"),
+                  /* %1$s is how much we've got,
+                     %2$s is the torrent's total size,
+                     %3$.2f%% is a percentage of the two,
+                     %4$s is how much we've uploaded,
+                     %5$s is our upload-to-download ratio */
+                  _("%1$s of %2$s (%3$.2f%%), uploaded %4$s (Ratio: %5$s)"),
                   tr_strlsize( buf1, haveTotal, sizeof(buf1) ),
                   tr_strlsize( buf2, info->totalSize, sizeof(buf2) ),
                   torStat->percentComplete * 100.0,
                   tr_strlsize( buf3, torStat->uploadedEver, sizeof(buf3) ),
-                  torStat->ratio );
+                  tr_strlratio( buf4, torStat->ratio, sizeof( buf4 ) ) );
     else
         str = g_strdup_printf(
-                  _("%s, uploaded %s (Ratio: %.1f)"),
+                  /* %1$s is the torrent's total size,
+                     %2$s is how much we've uploaded,
+                     %3$s is our upload-to-download ratio */
+                  _("%1$s, uploaded %2$s (Ratio: %3$s)"),
                   tr_strlsize( buf1, info->totalSize, sizeof(buf1) ),
                   tr_strlsize( buf2, torStat->uploadedEver, sizeof(buf2) ),
-                  torStat->ratio );
+                  tr_strlratio( buf3, torStat->ratio, sizeof( buf3 ) ) );
+
+    /* add time when downloading */
+    if( torStat->status == TR_STATUS_DOWNLOAD )
+    {
+        const int eta = torStat->eta;
+        GString * gstr = g_string_new( str );
+        g_string_append( gstr, " - " );
+        if( eta < 0 )
+            g_string_append( gstr, _( "Stalled" ) );
+        else {
+            char timestr[128];
+            tr_strltime( timestr, eta, sizeof( timestr ) );
+            /* time remaining */
+            g_string_append_printf( gstr, _( "%s remaining" ), timestr );
+        }
+        g_free( str );
+        str = g_string_free( gstr, FALSE );
+    }
 
     return str;
 }
@@ -107,8 +102,8 @@ static char*
 getShortTransferString( const tr_stat * torStat, char * buf, size_t buflen )
 {
     char downStr[32], upStr[32];
-    const int haveDown = ( torStat->rateDownload * 1024 ) > 1.0;
-    const int haveUp = ( torStat->rateUpload * 1024 ) > 1.0;
+    const int haveDown = torStat->peersSendingToUs > 0;
+    const int haveUp = torStat->peersGettingFromUs > 0;
 
     if( haveDown )
         tr_strlspeed( downStr, torStat->rateDownload, sizeof(downStr) );
@@ -116,12 +111,18 @@ getShortTransferString( const tr_stat * torStat, char * buf, size_t buflen )
         tr_strlspeed( upStr, torStat->rateUpload, sizeof(upStr) );
 
     if( haveDown && haveUp )
-        g_snprintf( buf, buflen, _( "Down: %s, Up: %s"), downStr, upStr );
+        /* Translators: do not translate the "speed|" disambiguation prefix.
+           %1$s is the download speed
+           %2$s is the upload speed */
+        g_snprintf( buf, buflen, Q_( "speed|Down: %1$s, Up: %2$s"), downStr, upStr );
     else if( haveDown )
+        /* download speed */
         g_snprintf( buf, buflen, _( "Down: %s" ), downStr );
     else if( haveUp )
+        /* upload speed */
         g_snprintf( buf, buflen, _( "Up: %s" ), upStr );
     else
+        /* the torrent isn't uploading or downloading */
         g_strlcpy( buf, _( "Idle" ), buflen );
 
     return buf;
@@ -139,19 +140,23 @@ getShortStatusString( const tr_stat * torStat )
             break;
 
         case TR_STATUS_CHECK_WAIT:
-            g_string_assign( gstr, _( "Waiting to Verify local data" ) );
+            g_string_assign( gstr, _( "Waiting to verify local data" ) );
             break;
 
         case TR_STATUS_CHECK:
             g_string_append_printf( gstr, _("Verifying local data (%.1f%% tested)"),
                                     torStat->recheckProgress * 100.0 );
+            break;
 
         case TR_STATUS_DOWNLOAD:
         case TR_STATUS_SEED:
         case TR_STATUS_DONE: {
             char buf[128];
-            if( torStat->status != TR_STATUS_DOWNLOAD )
-                g_string_append_printf( gstr, _("Ratio: %.1f, " ), torStat->ratio );
+            if( torStat->status != TR_STATUS_DOWNLOAD ) {
+                tr_strlratio( buf, torStat->ratio, sizeof( buf ) );
+                g_string_append_printf( gstr, _("Ratio: %s" ), buf );
+                g_string_append( gstr, ", " );
+            }
             getShortTransferString( torStat, buf, sizeof( buf ) );
             g_string_append( gstr, buf );
             break;
@@ -190,8 +195,8 @@ getStatusString( const tr_stat * torStat )
 
         case TR_STATUS_DOWNLOAD:
             g_string_append_printf( gstr,
-                ngettext( "Downloading from %d of %d connected peer",
-                          "Downloading from %d of %d connected peers",
+                ngettext( "Downloading from %1$'d of %2$'d connected peer",
+                          "Downloading from %1$'d of %2$'d connected peers",
                           torStat->peersConnected ),
                 torStat->peersSendingToUs,
                 torStat->peersConnected );
@@ -200,8 +205,8 @@ getStatusString( const tr_stat * torStat )
         case TR_STATUS_DONE:
         case TR_STATUS_SEED:
             g_string_append_printf( gstr,
-                ngettext( "Seeding to %d of %d connected peer",
-                          "Seeding to %d of %d connected peers",
+                ngettext( "Seeding to %1$'d of %2$'d connected peer",
+                          "Seeding to %1$'d of %2$'d connected peers",
                           torStat->peersConnected ),
                 torStat->peersGettingFromUs,
                 torStat->peersConnected );
@@ -229,17 +234,9 @@ struct TorrentCellRendererPrivate
     tr_torrent * tor;
     GtkCellRenderer * text_renderer;
     GtkCellRenderer * text_renderer_err;
+    GtkCellRenderer * progress_renderer;
     int bar_height;
     gboolean minimal;
-    gboolean show_unavailable;
-    gboolean gradient;
-    GdkColor color_paused[2];
-    GdkColor color_verified[2];
-    GdkColor color_verifying[2];
-    GdkColor color_missing[2];
-    GdkColor color_unwanted[2];
-    GdkColor color_unavailable[2];
-    GdkColor color_seeding[2];
 };
 
 static void
@@ -332,158 +329,6 @@ torrent_cell_renderer_get_size( GtkCellRenderer  * cell,
     }
 }
 
-static void
-fillRect( TorrentCellRenderer * self,
-          GdkGC               * gc,
-          GdkDrawable         * drawable,
-          const GdkRectangle  * area_in,
-          const GdkColor      * colors,
-          size_t                n_colors )
-{
-    const int drawGradient = self->priv->gradient && ( n_colors > 1 );
-    assert( n_colors==1 || n_colors==2 );
-
-    if( !drawGradient )
-    {
-        gdk_gc_set_rgb_fg_color( gc, colors );
-        gdk_draw_rectangle( drawable, gc, TRUE,
-                            area_in->x, area_in->y,
-                            area_in->width, area_in->height );
-    }
-    else
-    {
-        int i;
-        const int steps = area_in->height;
-        const int step_height = area_in->height / steps;
-        const int r_inc = ((int)colors[1].red   - (int)colors[0].red) / steps;
-        const int g_inc = ((int)colors[1].green - (int)colors[0].green) / steps;
-        const int b_inc = ((int)colors[1].blue  - (int)colors[0].blue) / steps;
-
-        GdkRectangle area = *area_in;
-        GdkColor color = colors[0];
-     
-        area.height = step_height;
-        for( i=0; i<steps; ++i ) {
-            gdk_gc_set_rgb_fg_color( gc, &color );
-            gdk_draw_rectangle( drawable, gc, TRUE,
-                                area.x, area.y, area.width, area.height );
-            area.y += step_height;
-            color.red   += r_inc;
-            color.green += g_inc;
-            color.blue  += b_inc;
-        }
-    }
-}
-
-static void
-drawRegularBar( TorrentCellRenderer * self,
-                const tr_info       * info,
-                const tr_stat       * torStat,
-                GdkDrawable         * drawable,
-                GtkWidget           * widget,
-                const GdkRectangle  * area )
-{
-#if 1
-    const double verified = torStat->haveValid / (double)info->totalSize;
-    const double unverified = torStat->haveUnchecked / (double)info->totalSize;
-    const double unavailable = ( torStat->desiredSize
-                       - torStat->desiredAvailable ) / (double)info->totalSize;
-    const double unwanted = ( info->totalSize
-                            - torStat->desiredSize ) / (double)info->totalSize;
-#else /* for testing */
-    const double verified = 0.5;
-    const double unverified = 0.1;
-    const double unavailable = 0.1;
-    const double unwanted = 0.1;
-#endif
-    const double missing = 1.0 - verified - unverified - unavailable - unwanted;
-    const int verifiedWidth = (int)( verified * area->width );
-    const int unverifiedWidth = (int)( unverified * area->width );
-    const int unavailableWidth = (int)( unavailable * area->width );
-    const int unwantedWidth = (int)( unwanted * area->width );
-    const int missingWidth = (int)( missing * area->width );
-
-    const gboolean isActive = torStat->status == TR_STATUS_DOWNLOAD
-                           || torStat->status == TR_STATUS_DONE
-                           || torStat->status == TR_STATUS_SEED;
-    const gboolean isChecking = torStat->status == TR_STATUS_CHECK
-                             || torStat->status == TR_STATUS_CHECK_WAIT;
-
-    int x = area->x;
-    int w = 0;
-    GdkGC * gc = gdk_gc_new( drawable );
-    GdkRectangle rect = *area;
-
-    if(( w = verifiedWidth )) {
-        const GdkColor * colors;
-        if( !isActive )
-            colors = self->priv->color_paused;
-        else if( torStat->status == TR_STATUS_DOWNLOAD )
-            colors = self->priv->color_verified;
-        else
-            colors = self->priv->color_seeding;
-        rect.x = x;
-        rect.width = w;
-        fillRect( self, gc, drawable, &rect, colors, 2 );
-        x += w;
-    }
-
-    if(( w = unverifiedWidth )) {
-        const GdkColor * colors = isActive ? self->priv->color_verifying
-                                           : self->priv->color_paused;
-        rect.x = x;
-        rect.width = w;
-        fillRect( self, gc, drawable, &rect, colors, 2 );
-        x += w;
-    }
-
-    if(( w = missingWidth )) {
-        rect.x = x;
-        rect.width = w;
-        fillRect( self, gc, drawable, &rect, self->priv->color_missing, 2 );
-        x += w;
-    }
-
-    if(( w = unwantedWidth )) {
-        rect.x = x;
-        rect.width = w;
-        fillRect( self, gc, drawable, &rect, self->priv->color_unwanted, 2 );
-        x += w;
-    }
-
-    if(( w = unavailableWidth )) {
-        const GdkColor * colors = isActive && self->priv->show_unavailable
-                                ? self->priv->color_unavailable
-                                : self->priv->color_missing;
-        rect.x = x;
-        rect.width = w;
-        fillRect( self, gc, drawable, &rect, colors, 2 );
-        x += w;
-    }
-
-    if( isChecking ) {
-        const int checkedWidth = torStat->recheckProgress * area->width;
-        const int h2 = area->height / 2;
-        rect = *area;
-        rect.y += h2;
-        rect.height -= h2;
-        fillRect( self, gc, drawable, &rect, self->priv->color_missing, 2 );
-        rect.width = checkedWidth;
-        fillRect( self, gc, drawable, &rect, self->priv->color_verifying, 2 );
-    }
-
-    gtk_paint_shadow( gtk_widget_get_style( widget ),
-                      drawable,
-                      GTK_STATE_NORMAL,
-                      GTK_SHADOW_IN,
-                      NULL,
-                      widget,
-                      NULL,
-                      area->x, area->y, area->width, area->height );
-
-    gdk_gc_unref( gc );
-}
-
 static void
 torrent_cell_renderer_render( GtkCellRenderer      * cell,
                               GdkDrawable          * window,
@@ -494,6 +339,12 @@ torrent_cell_renderer_render( GtkCellRenderer      * cell,
                               GtkCellRendererState   flags)
 {
     TorrentCellRenderer * self = TORRENT_CELL_RENDERER( cell );
+
+#ifdef TEST_RTL
+    GtkTextDirection real_dir = gtk_widget_get_direction( widget );
+    gtk_widget_set_direction( widget, GTK_TEXT_DIR_RTL );
+#endif
+
     if( self && self->priv->tor )
     {
         const tr_torrent * tor = self->priv->tor;
@@ -509,6 +360,7 @@ torrent_cell_renderer_render( GtkCellRenderer      * cell,
         GtkCellRenderer * text_renderer = torStat->error != 0
             ? p->text_renderer_err
             : p->text_renderer;
+        const gboolean isActive = torStat->status != TR_STATUS_STOPPED;
 
         g_object_get( self, "xpad", &xpad, "ypad", &ypad, NULL );
 
@@ -518,6 +370,9 @@ torrent_cell_renderer_render( GtkCellRenderer      * cell,
         my_bg.width -= xpad*2;
         my_cell = my_expose = my_bg;
 
+        g_object_set( text_renderer, "sensitive", isActive, NULL );
+        g_object_set( p->progress_renderer, "sensitive", isActive, NULL );
+
         /* above the progressbar */
         if( !p->minimal )
         {
@@ -601,7 +456,18 @@ torrent_cell_renderer_render( GtkCellRenderer      * cell,
 
         /* the progressbar */
         my_cell.height = p->bar_height;
-        drawRegularBar( self, info, torStat, window, widget, &my_cell );
+        if( 1 )
+        {
+            const double havePercent = ( torStat->haveValid + torStat->haveUnchecked )
+                                                              / (double)info->totalSize;
+            g_object_set( p->progress_renderer, "value", (int)(havePercent*100.0), 
+                                                "text", "",
+                                                NULL );
+            gtk_cell_renderer_render( p->progress_renderer,
+                                      window, widget,
+                                      &my_cell, &my_cell, &my_cell, flags );
+ 
+        }
         my_bg.y     += my_cell.height;
         my_cell.y   += my_cell.height;
         my_expose.y += my_cell.height;
@@ -628,12 +494,10 @@ torrent_cell_renderer_render( GtkCellRenderer      * cell,
             g_free( statusString );
         }
     }
-}
 
-static void
-v2c( GdkColor * color, const GValue * value )
-{
-    gdk_color_parse( g_value_get_string( value ), color );
+#ifdef TEST_RTL
+    gtk_widget_set_direction( widget, real_dir );
+#endif
 }
 
 static void
@@ -647,43 +511,15 @@ torrent_cell_renderer_set_property( GObject      * object,
 
     switch( property_id )
     {
-        case P_COLOR_MISSING:       v2c( &p->color_missing[0],     v ); break;
-        case P_COLOR_MISSING_2:     v2c( &p->color_missing[1],     v ); break;
-        case P_COLOR_UNWANTED:      v2c( &p->color_unwanted[0],    v ); break;
-        case P_COLOR_UNWANTED_2:    v2c( &p->color_unwanted[1],    v ); break;
-        case P_COLOR_PAUSED:        v2c( &p->color_paused[0],      v ); break;
-        case P_COLOR_PAUSED_2:      v2c( &p->color_paused[1],      v ); break;
-        case P_COLOR_VERIFIED:      v2c( &p->color_verified[0],    v ); break;
-        case P_COLOR_VERIFIED_2:    v2c( &p->color_verified[1],    v ); break;
-        case P_COLOR_UNAVAILABLE:   v2c( &p->color_unavailable[0], v ); break;
-        case P_COLOR_UNAVAILABLE_2: v2c( &p->color_unavailable[1], v ); break;
-        case P_COLOR_VERIFYING:     v2c( &p->color_verifying[0],   v ); break;
-        case P_COLOR_VERIFYING_2:   v2c( &p->color_verifying[1],   v ); break;
-        case P_COLOR_SEEDING:       v2c( &p->color_seeding[0],     v ); break;
-        case P_COLOR_SEEDING_2:     v2c( &p->color_seeding[1],     v ); break;
         case P_TORRENT:     p->tor = g_value_get_pointer( v ); break;
         case P_BAR_HEIGHT:  p->bar_height = g_value_get_int( v ); break;
         case P_MINIMAL:     p->minimal  = g_value_get_boolean( v ); break;
-        case P_GRADIENT:    p->gradient = g_value_get_boolean( v ); break;
-        case P_SHOW_UNAVAILABLE:
-                            p->show_unavailable = g_value_get_boolean( v ); break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec );
             break;
     }
 }
 
-static void
-c2v( GValue * value, const GdkColor * color )
-{
-    char buf[16];
-    g_snprintf( buf, sizeof(buf), "#%2.2x%2.2x%2.2x",
-                (color->red >> 8) & 0xff,
-                (color->green >> 8) & 0xff,
-                (color->blue >> 8) & 0xff );
-    g_value_set_string( value, buf );
-}
-
 static void
 torrent_cell_renderer_get_property( GObject      * object,
                                     guint          property_id,
@@ -695,26 +531,9 @@ torrent_cell_renderer_get_property( GObject      * object,
 
     switch( property_id )
     {
-        case P_COLOR_MISSING:       c2v( v, &p->color_missing[0] ); break;
-        case P_COLOR_MISSING_2:     c2v( v, &p->color_missing[1] ); break;
-        case P_COLOR_UNWANTED:      c2v( v, &p->color_unwanted[0] ); break;
-        case P_COLOR_UNWANTED_2:    c2v( v, &p->color_unwanted[1] ); break;
-        case P_COLOR_PAUSED:        c2v( v, &p->color_paused[0] ); break;
-        case P_COLOR_PAUSED_2:      c2v( v, &p->color_paused[1] ); break;
-        case P_COLOR_VERIFIED:      c2v( v, &p->color_verified[0] ); break;
-        case P_COLOR_VERIFIED_2:    c2v( v, &p->color_verified[1] ); break;
-        case P_COLOR_UNAVAILABLE:   c2v( v, &p->color_unavailable[0] ); break;
-        case P_COLOR_UNAVAILABLE_2: c2v( v, &p->color_unavailable[1] ); break;
-        case P_COLOR_VERIFYING:     c2v( v, &p->color_verifying[0] ); break;
-        case P_COLOR_VERIFYING_2:   c2v( v, &p->color_verifying[1] ); break;
-        case P_COLOR_SEEDING:       c2v( v, &p->color_seeding[0] ); break;
-        case P_COLOR_SEEDING_2:     c2v( v, &p->color_seeding[1] ); break;
         case P_TORRENT:     g_value_set_pointer( v, p->tor ); break;
         case P_BAR_HEIGHT:  g_value_set_int( v, p->bar_height ); break;
         case P_MINIMAL:     g_value_set_boolean( v, p->minimal ); break;
-        case P_GRADIENT:    g_value_set_boolean( v, p->gradient ); break;
-        case P_SHOW_UNAVAILABLE:
-                            g_value_set_boolean( v, p->show_unavailable ); break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec );
             break;
@@ -725,12 +544,18 @@ static void
 torrent_cell_renderer_dispose( GObject * o )
 {
     TorrentCellRenderer * r = TORRENT_CELL_RENDERER( o );
+    GObjectClass * parent;
+
     if( r && r->priv )
     {
         g_object_unref( G_OBJECT( r->priv->text_renderer ) );
         g_object_unref( G_OBJECT( r->priv->text_renderer_err ) );
+        g_object_unref( G_OBJECT( r->priv->progress_renderer ) );
         r->priv = NULL;
     }
+
+    parent = g_type_class_peek( g_type_parent( TORRENT_CELL_RENDERER_TYPE ) );
+    parent->dispose( o );
 }
 
 static void
@@ -761,70 +586,6 @@ torrent_cell_renderer_class_init( TorrentCellRendererClass * klass )
     g_object_class_install_property( gobject_class, P_MINIMAL,
         g_param_spec_boolean( "minimal", NULL, "Minimal Mode",
                               FALSE, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_GRADIENT,
-        g_param_spec_boolean( "gradient", NULL, "Render Progress as a Gradient",
-                              TRUE, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_SHOW_UNAVAILABLE,
-        g_param_spec_boolean( "unavailable", NULL, "Show Unavailable",
-                              FALSE, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_MISSING,
-        g_param_spec_string( "missing-color", NULL, "Color for Missing Data",
-                             DEFAULT_COLOR_MISSING, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_MISSING,
-        g_param_spec_string( "missing-color-2", NULL, "Gradient Color for Missing Data",
-                             DEFAULT_COLOR_MISSING_2, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_UNWANTED,
-        g_param_spec_string( "unwanted-color", NULL, "Color for Unwanted Data",
-                             DEFAULT_COLOR_UNWANTED, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_UNWANTED_2,
-        g_param_spec_string( "unwanted-color-2", NULL, "Gradient Color for Unwanted Data",
-                             DEFAULT_COLOR_UNWANTED_2, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_PAUSED,
-        g_param_spec_string( "paused-color", NULL, "Color for Paused Data",
-                             DEFAULT_COLOR_PAUSED, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_PAUSED_2,
-        g_param_spec_string( "paused-color-2", NULL, "Gradient Color for Paused Data",
-                             DEFAULT_COLOR_PAUSED_2, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_VERIFIED,
-        g_param_spec_string( "verified-color", NULL, "Color for Verified Data",
-                             DEFAULT_COLOR_VERIFIED, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_VERIFIED_2,
-        g_param_spec_string( "verified-color-2", NULL, "Gradient Color for Verified Data",
-                             DEFAULT_COLOR_VERIFIED_2, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_UNAVAILABLE,
-        g_param_spec_string( "unavailable-color", NULL, "Color for Unavailable Data",
-                             DEFAULT_COLOR_UNAVAILABLE, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_UNAVAILABLE_2,
-        g_param_spec_string( "unavailable-color-2", NULL, "Gradient Color for Unavailable Data",
-                             DEFAULT_COLOR_UNAVAILABLE_2, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_VERIFYING,
-        g_param_spec_string( "verifying-color", NULL, "Color for Verifying Data",
-                             DEFAULT_COLOR_VERIFYING, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_VERIFYING_2,
-        g_param_spec_string( "verifying-color-2", NULL, "Gradient Color for Verifying Data",
-                             DEFAULT_COLOR_VERIFYING_2, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_SEEDING,
-        g_param_spec_string( "seeding-color", NULL, "Color for Seeding Data",
-                             DEFAULT_COLOR_SEEDING, G_PARAM_READWRITE ) );
-
-    g_object_class_install_property( gobject_class, P_COLOR_SEEDING_2,
-        g_param_spec_string( "seeding-color-2", NULL, "Second Color for Seeding Data",
-                             DEFAULT_COLOR_SEEDING_2, G_PARAM_READWRITE ) );
 }
 
 static void
@@ -840,28 +601,13 @@ torrent_cell_renderer_init( GTypeInstance * instance, gpointer g_class UNUSED )
     p->tor = NULL;
     p->text_renderer = gtk_cell_renderer_text_new( );
     p->text_renderer_err = gtk_cell_renderer_text_new(  );
+    p->progress_renderer = gtk_cell_renderer_progress_new(  );
     g_object_set( p->text_renderer_err, "foreground", "red", NULL );
     tr_object_ref_sink( p->text_renderer );
     tr_object_ref_sink( p->text_renderer_err );
+    tr_object_ref_sink( p->progress_renderer );
 
-    p->gradient = TRUE;
-    p->show_unavailable = TRUE;
     p->bar_height = DEFAULT_BAR_HEIGHT;
-
-    gdk_color_parse( DEFAULT_COLOR_VERIFIED,      &p->color_verified[0] );
-    gdk_color_parse( DEFAULT_COLOR_VERIFIED_2,    &p->color_verified[1] );
-    gdk_color_parse( DEFAULT_COLOR_MISSING,       &p->color_missing[0] );
-    gdk_color_parse( DEFAULT_COLOR_MISSING_2,     &p->color_missing[1] );
-    gdk_color_parse( DEFAULT_COLOR_UNWANTED,      &p->color_unwanted[0] );
-    gdk_color_parse( DEFAULT_COLOR_UNWANTED_2,    &p->color_unwanted[1] );
-    gdk_color_parse( DEFAULT_COLOR_UNAVAILABLE,   &p->color_unavailable[0] );
-    gdk_color_parse( DEFAULT_COLOR_UNAVAILABLE_2, &p->color_unavailable[1] );
-    gdk_color_parse( DEFAULT_COLOR_VERIFYING,     &p->color_verifying[0] );
-    gdk_color_parse( DEFAULT_COLOR_VERIFYING_2,   &p->color_verifying[1] );
-    gdk_color_parse( DEFAULT_COLOR_SEEDING,       &p->color_seeding[0] );
-    gdk_color_parse( DEFAULT_COLOR_SEEDING_2,     &p->color_seeding[1] );
-    gdk_color_parse( DEFAULT_COLOR_PAUSED,        &p->color_paused[0] );
-    gdk_color_parse( DEFAULT_COLOR_PAUSED_2,      &p->color_paused[1] );
 }
 
 GType
