@@ -7,14 +7,13 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.c 4404 2008-01-01 17:20:20Z charles $
+ * $Id: makemeta.c 5221 2008-03-07 20:48:36Z charles $
  */
 
 #include <assert.h>
 #include <errno.h>
-#include <math.h>
 #include <stdio.h> /* FILE, snprintf, stderr */
-#include <stdlib.h>
+#include <stdlib.h> /* qsort */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -57,7 +56,7 @@ getFiles( const char        * dir,
     tr_buildPath( buf, sizeof(buf), dir, base, NULL );
     i = stat( buf, &sb );
     if( i ) {
-        tr_err("makemeta couldn't stat \"%s\"; skipping. (%s)", buf, strerror(errno));
+        tr_err( _( "Torrent Creator is skipping file \"%s\": %s" ), buf, tr_strerror(errno));
         return list;
     }
 
@@ -73,7 +72,10 @@ getFiles( const char        * dir,
     {
         struct FileList * node = tr_new( struct FileList, 1 );
         node->size = sb.st_size;
-        node->filename = tr_strdup( buf );
+        if( ( buf[0]=='.' ) && ( buf[1]=='/' ) )
+            node->filename = tr_strdup( buf + 2 );
+        else
+            node->filename = tr_strdup( buf );
         node->next = list;
         list = node;
     }
@@ -202,6 +204,12 @@ getHashInfo ( tr_metainfo_builder * b )
     b->pieceIndex = 0;
     totalRemain = b->totalSize;
     fp = fopen( b->files[fileIndex].filename, "rb" );
+    if( !fp ) {
+        tr_err( _( "Couldn't open \"%s\": %s" ), b->files[fileIndex].filename, tr_strerror( errno ) );
+        tr_free( ret );
+        b->failed = 1;
+        return NULL;
+    }
     while ( totalRemain )
     {
         uint8_t *bufptr = buf;
@@ -225,6 +233,12 @@ getHashInfo ( tr_metainfo_builder * b )
                 fp = NULL;
                 if( ++fileIndex < b->fileCount ) {
                     fp = fopen( b->files[fileIndex].filename, "rb" );
+                    if( !fp ) {
+                        tr_err( _( "Couldn't open \"%s\": %s" ), b->files[fileIndex].filename, tr_strerror( errno ) );
+                        tr_free( ret );
+                        b->failed = 1;
+                        return NULL;
+                    }
                 }
             }
         }
@@ -256,10 +270,10 @@ getHashInfo ( tr_metainfo_builder * b )
 static void
 getFileInfo( const char                      * topFile,
              const tr_metainfo_builder_file  * file,
-             benc_val_t                      * uninitialized_length,
-             benc_val_t                      * uninitialized_path )
+             tr_benc                         * uninitialized_length,
+             tr_benc                         * uninitialized_path )
 {
-    benc_val_t *sub;
+    tr_benc *sub;
     const char *pch, *prev;
     const size_t topLen = strlen(topFile) + 1; /* +1 for '/' */
     int n;
@@ -294,7 +308,7 @@ getFileInfo( const char                      * topFile,
 }
 
 static void
-makeFilesList( benc_val_t                 * list,
+makeFilesList( tr_benc                    * list,
                const tr_metainfo_builder  * builder )
 {
     uint32_t i = 0;
@@ -303,8 +317,8 @@ makeFilesList( benc_val_t                 * list,
 
     for( i=0; i<builder->fileCount; ++i )
     {
-        benc_val_t * dict = tr_bencListAdd( list );
-        benc_val_t *length, *pathVal;
+        tr_benc * dict = tr_bencListAdd( list );
+        tr_benc *length, *pathVal;
 
         tr_bencInit( dict, TYPE_DICT );
         tr_bencDictReserve( dict, 2 );
@@ -315,11 +329,11 @@ makeFilesList( benc_val_t                 * list,
 }
 
 static void
-makeInfoDict ( benc_val_t           * dict,
+makeInfoDict ( tr_benc              * dict,
                tr_metainfo_builder  * builder )
 {
     uint8_t * pch;
-    benc_val_t * val;
+    tr_benc * val;
     char base[MAX_PATH_LENGTH];
 
     tr_bencDictReserve( dict, 5 );
@@ -343,9 +357,10 @@ makeInfoDict ( benc_val_t           * dict,
     val = tr_bencDictAdd( dict, "piece length" );
     tr_bencInitInt( val, builder->pieceSize );
 
-    pch = getHashInfo( builder );
-    val = tr_bencDictAdd( dict, "pieces" );
-    tr_bencInitStr( val, pch, SHA_DIGEST_LENGTH * builder->pieceCount, 0 );
+    if( ( pch = getHashInfo( builder ) ) ) {
+        val = tr_bencDictAdd( dict, "pieces" );
+        tr_bencInitStr( val, pch, SHA_DIGEST_LENGTH * builder->pieceCount, 0 );
+    }
 
     val = tr_bencDictAdd( dict, "private" );
     tr_bencInitInt( val, builder->isPrivate ? 1 : 0 );
@@ -354,7 +369,7 @@ makeInfoDict ( benc_val_t           * dict,
 static void tr_realMakeMetaInfo ( tr_metainfo_builder * builder )
 {
     int n = 5;
-    benc_val_t top, *val;
+    tr_benc top, *val;
 
     tr_bencInit ( &top, TYPE_DICT );
     if ( builder->comment && *builder->comment ) ++n;
@@ -383,7 +398,7 @@ static void tr_realMakeMetaInfo ( tr_metainfo_builder * builder )
     makeInfoDict( val, builder );
 
     /* save the file */
-    if ( !builder->abortFlag ) {
+    if ( !builder->failed && !builder->abortFlag ) {
         size_t nmemb;
         char * pch = tr_bencSave( &top, &n );
         FILE * fp = fopen( builder->outputFile, "wb+" );
