@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: metainfo.c 4404 2008-01-01 17:20:20Z charles $
+ * $Id: metainfo.c 5313 2008-03-19 20:07:27Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,6 +23,7 @@
  *****************************************************************************/
 
 #include <assert.h>
+#include <ctype.h> /* isspace */
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,6 +39,7 @@
 #include "crypto.h" /* tr_sha1 */
 #include "metainfo.h"
 #include "platform.h"
+#include "trcompat.h" /* strlcpy */
 #include "utils.h"
 
 
@@ -67,10 +69,10 @@ tr_httpParseUrl( const char * url_in, int len,
 /***********************************************************************
  * Local prototypes
  **********************************************************************/
-static int getannounce( tr_info * inf, benc_val_t * meta );
+static int getannounce( tr_info * inf, tr_benc * meta );
 static char * announceToScrape( const char * announce );
-static int parseFiles( tr_info * inf, benc_val_t * name,
-                       benc_val_t * files, benc_val_t * length );
+static int parseFiles( tr_info * inf, tr_benc * name,
+                       tr_benc * files, tr_benc * length );
 
 /***
 ****
@@ -167,11 +169,12 @@ savedname( char * name, size_t len, const char * hash, const char * tag )
 
 
 int
-tr_metainfoParse( tr_info * inf, const benc_val_t * meta_in, const char * tag )
+tr_metainfoParse( tr_info * inf, const tr_benc * meta_in, const char * tag )
 {
     int i;
-    benc_val_t * beInfo, * val, * val2;
-    benc_val_t * meta = (benc_val_t *) meta_in;
+    tr_benc * beInfo, * val, * val2;
+    tr_benc * meta = (tr_benc *) meta_in;
+    char buf[4096];
 
     /* info_hash: urlencoded 20-byte SHA1 hash of the value of the info key
      * from the Metainfo file. Note that the value will be a bencoded 
@@ -185,30 +188,28 @@ tr_metainfoParse( tr_info * inf, const benc_val_t * meta_in, const char * tag )
     }
     else
     {
-        tr_err( "info dictionary not found!" );
+        tr_err( _( "Missing metadata entry \"%s\"" ), "info" );
         return TR_EINVALID;
     }
 
-    for( i = 0; i < SHA_DIGEST_LENGTH; i++ )
-    {
-        snprintf( inf->hashString + i * 2, sizeof( inf->hashString ) - i * 2,
-                  "%02x", inf->hash[i] );
-    }
+    tr_sha1_to_hex( inf->hashString, inf->hash );
     savedname( inf->torrent, sizeof( inf->torrent ), inf->hashString, tag );
 
-    /* Comment info */
+    /* comment */
+    memset( buf, '\0', sizeof( buf ) );
     val = tr_bencDictFindFirst( meta, "comment.utf-8", "comment", NULL );
-    if( NULL != val && TYPE_STR == val->type )
-    {
-        strlcat_utf8( inf->comment, val->val.s.s, sizeof( inf->comment ), 0 );
-    }
+    if( val && val->type == TYPE_STR )
+        strlcat_utf8( buf, val->val.s.s, sizeof( buf ), 0 );
+    tr_free( inf->comment );
+    inf->comment = tr_strdup( buf );
     
-    /* Creator info */
+    /* creator */
+    memset( buf, '\0', sizeof( buf ) );
     val = tr_bencDictFindFirst( meta, "created by.utf-8", "created by", NULL );
-    if( NULL != val && TYPE_STR == val->type )
-    {
-        strlcat_utf8( inf->creator, val->val.s.s, sizeof( inf->creator ), 0 );
-    }
+    if( val && val->type == TYPE_STR )
+        strlcat_utf8( buf, val->val.s.s, sizeof( buf ), 0 );
+    tr_free( inf->creator );
+    inf->creator = tr_strdup( buf );
     
     /* Date created */
     inf->dateCreated = 0;
@@ -231,7 +232,10 @@ tr_metainfoParse( tr_info * inf, const benc_val_t * meta_in, const char * tag )
     val = tr_bencDictFind( beInfo, "piece length" );
     if( NULL == val || TYPE_INT != val->type )
     {
-        tr_err( "%s \"piece length\" entry", ( val ? "Invalid" : "Missing" ) );
+        if( val )
+            tr_err( _( "Invalid metadata entry \"%s\"" ), "piece length" );
+        else
+            tr_err( _( "Missing metadata entry \"%s\"" ), "piece length" );
         goto fail;
     }
     inf->pieceSize = val->val.i;
@@ -240,12 +244,15 @@ tr_metainfoParse( tr_info * inf, const benc_val_t * meta_in, const char * tag )
     val = tr_bencDictFind( beInfo, "pieces" );
     if( NULL == val || TYPE_STR != val->type )
     {
-        tr_err( "%s \"pieces\" entry", ( val ? "Invalid" : "Missing" ) );
+        if( val )
+            tr_err( _( "Invalid metadata entry \"%s\"" ), "pieces" );
+        else
+            tr_err( _( "Missing metadata entry \"%s\"" ), "pieces" );
         goto fail;
     }
     if( val->val.s.i % SHA_DIGEST_LENGTH )
     {
-        tr_err( "Invalid \"piece\" string (size is %d)", val->val.s.i );
+        tr_err( _( "Invalid metadata entry \"%s\"" ), "pieces" );
         goto fail;
     }
     inf->pieceCount = val->val.s.i / SHA_DIGEST_LENGTH;
@@ -267,15 +274,9 @@ tr_metainfoParse( tr_info * inf, const benc_val_t * meta_in, const char * tag )
         goto fail;
     }
 
-    if( !inf->fileCount )
+    if( !inf->fileCount || !inf->totalSize )
     {
-        tr_err( "Torrent has no files." );
-        goto fail;
-    }
-
-    if( !inf->totalSize )
-    {
-        tr_err( "Torrent is zero bytes long." );
+        tr_err( _( "Torrent is corrupt" ) ); /* the content is missing! */
         goto fail;
     }
 
@@ -284,7 +285,7 @@ tr_metainfoParse( tr_info * inf, const benc_val_t * meta_in, const char * tag )
     if( (uint64_t) inf->pieceCount !=
         ( inf->totalSize + inf->pieceSize - 1 ) / inf->pieceSize )
     {
-        tr_err( "Size of hashes and files don't match" );
+        tr_err( _( "Torrent is corrupt" ) ); /* size of hashes and files don't match */
         goto fail;
     }
 
@@ -305,8 +306,13 @@ void tr_metainfoFree( tr_info * inf )
 {
     int i, j;
 
+    for( i=0; i<inf->fileCount; ++i )
+        tr_free( inf->files[i].name );
+
     tr_free( inf->pieces );
     tr_free( inf->files );
+    tr_free( inf->comment );
+    tr_free( inf->creator );
     tr_free( inf->primaryAddress );
     
     for( i=0; i<inf->trackerTiers; ++i ) {
@@ -319,12 +325,13 @@ void tr_metainfoFree( tr_info * inf )
     memset( inf, '\0', sizeof(tr_info) );
 }
 
-static int getfile( char * buf, int size,
-                    const char * prefix, benc_val_t * name )
+static int
+getfile( char ** setme, const char * prefix, tr_benc * name )
 {
-    benc_val_t  * dir;
+    tr_benc     * dir;
     const char ** list;
     int           ii, jj;
+    char          buf[4096];
 
     if( TYPE_LIST != name->type )
     {
@@ -364,20 +371,24 @@ static int getfile( char * buf, int size,
         return TR_EINVALID;
     }
 
-    strlcat_utf8( buf, prefix, size, 0 );
+    memset( buf, 0, sizeof( buf ) );
+    strlcat_utf8( buf, prefix, sizeof(buf), 0 );
     for( ii = 0; jj > ii; ii++ )
     {
-        strlcat_utf8( buf, TR_PATH_DELIMITER_STR, size, 0 );
-        strlcat_utf8( buf, list[ii], size, TR_PATH_DELIMITER );
+        strlcat_utf8( buf, TR_PATH_DELIMITER_STR, sizeof(buf), 0 );
+        strlcat_utf8( buf, list[ii], sizeof(buf), TR_PATH_DELIMITER );
     }
     free( list );
 
+    tr_free( *setme );
+    *setme = tr_strdup( buf );
+
     return TR_OK;
 }
 
-static int getannounce( tr_info * inf, benc_val_t * meta )
+static int getannounce( tr_info * inf, tr_benc * meta )
 {
-    benc_val_t        * val, * subval, * urlval;
+    tr_benc           * val, * subval, * urlval;
     char              * address, * announce;
     int                 ii, jj, port, random, subcount;
     tr_tracker_info   * sublist;
@@ -455,7 +466,7 @@ static int getannounce( tr_info * inf, benc_val_t * meta )
         /* did we use any of the tiers? */
         if( 0 == inf->trackerTiers )
         {
-            tr_inf( "No valid entries in \"announce-list\"" );
+            tr_inf( _( "Invalid metadata entry \"%s\"" ), "announce-list" );
             free( inf->trackerList );
             inf->trackerList = NULL;
         }
@@ -475,17 +486,21 @@ static int getannounce( tr_info * inf, benc_val_t * meta )
     val = tr_bencDictFind( meta, "announce" );
     if( NULL == val || TYPE_STR != val->type )
     {
-        tr_err( "No \"announce\" entry" );
+        tr_err( _( "Missing metadata entry \"%s\"" ), "announce" );
         return TR_EINVALID;
     }
 
     if( !inf->trackerTiers )
     {
+        char buf[4096], *pch;
+        strlcpy( buf, val->val.s.s, sizeof( buf ) );
+        pch = buf;
+        while( isspace( *pch ) )
+            ++pch;
 
-        if( tr_httpParseUrl( val->val.s.s, val->val.s.i,
-                             &address, &port, &announce ) )
+        if( tr_httpParseUrl( pch, -1, &address, &port, &announce ) )
         {
-            tr_err( "Invalid announce URL (%s)", val->val.s.s );
+            tr_err( _( "Invalid announce URL \"%s\"" ), val->val.s.s );
             return TR_EINVALID;
         }
         sublist                   = calloc( 1, sizeof( sublist[0] ) );
@@ -595,13 +610,13 @@ tr_metainfoSave( const char * hash, const char * tag,
     file = fopen( path, "wb+" );
     if( !file )
     {
-        tr_err( "Could not open file (%s) (%s)", path, strerror( errno ) );
+        tr_err( _( "Couldn't open \"%s\": %s" ), path, tr_strerror( errno ) );
         return TR_EINVALID;
     }
     fseek( file, 0, SEEK_SET );
     if( fwrite( buf, 1, buflen, file ) != buflen )
     {
-        tr_err( "Could not write file (%s) (%s)", path, strerror( errno ) );
+        tr_err( _( "Couldn't save file \"%s\": %s" ), path, tr_strerror( errno ) );
         fclose( file );
         return TR_EINVALID;
     }
@@ -611,15 +626,18 @@ tr_metainfoSave( const char * hash, const char * tag,
 }
 
 static int
-parseFiles( tr_info * inf, benc_val_t * name,
-            benc_val_t * files, benc_val_t * length )
+parseFiles( tr_info * inf, tr_benc * name,
+            tr_benc * files, tr_benc * length )
 {
-    benc_val_t * item, * path;
+    tr_benc * item, * path;
     int ii;
 
     if( NULL == name || TYPE_STR != name->type )
     {
-        tr_err( "%s \"name\" string", ( name ? "Invalid" : "Missing" ) );
+        if( name )
+            tr_err( _( "Invalid metadata entry \"%s\"" ), "name" );
+        else
+            tr_err( _( "Missing metadata entry \"%s\"" ), "name" );
         return TR_EINVALID;
     }
 
@@ -627,7 +645,7 @@ parseFiles( tr_info * inf, benc_val_t * name,
                   TR_PATH_DELIMITER );
     if( '\0' == inf->name[0] )
     {
-        tr_err( "Invalid \"name\" string" );
+        tr_err( _( "Invalid metadata entry \"%s\"" ), "name" );
         return TR_EINVALID;
     }
     inf->totalSize = 0;
@@ -639,27 +657,28 @@ parseFiles( tr_info * inf, benc_val_t * name,
         inf->fileCount = files->val.l.count;
         inf->files     = calloc( inf->fileCount, sizeof( inf->files[0] ) );
 
-        if( NULL == inf->files )
-        {
+        if( !inf->files )
             return TR_EINVALID;
-        }
 
         for( ii = 0; files->val.l.count > ii; ii++ )
         {
             item = &files->val.l.vals[ii];
             path = tr_bencDictFindFirst( item, "path.utf-8", "path", NULL );
-            if( getfile( inf->files[ii].name, sizeof( inf->files[0].name ),
-                         inf->name, path ) )
+            if( getfile( &inf->files[ii].name, inf->name, path ) )
             {
-                tr_err( "%s \"path\" entry",
-                        ( path ? "Invalid" : "Missing" ) );
+                if( path )
+                    tr_err( _( "Invalid metadata entry \"%s\"" ), "path" );
+                else
+                    tr_err( _( "Missing metadata entry \"%s\"" ), "path" );
                 return TR_EINVALID;
             }
             length = tr_bencDictFind( item, "length" );
             if( NULL == length || TYPE_INT != length->type )
             {
-                tr_err( "%s \"length\" entry",
-                        ( length ? "Invalid" : "Missing" ) );
+                if( length )
+                    tr_err( _( "Invalid metadata entry \"%s\"" ), "length" );
+                else
+                    tr_err( _( "Missing metadata entry \"%s\"" ), "length" );
                 return TR_EINVALID;
             }
             inf->files[ii].length = length->val.i;
@@ -668,27 +687,27 @@ parseFiles( tr_info * inf, benc_val_t * name,
     }
     else if( NULL != length && TYPE_INT == length->type )
     {
+        char buf[4096];
+
         /* Single-file mode */
         inf->isMultifile = 0;
         inf->fileCount = 1;
         inf->files     = calloc( 1, sizeof( inf->files[0] ) );
 
-        if( NULL == inf->files )
-        {
+        if( !inf->files )
             return TR_EINVALID;
-        }
 
-        strlcat_utf8( inf->files[0].name, name->val.s.s,
-                      sizeof( inf->files[0].name ), TR_PATH_DELIMITER );
+        memset( buf, 0, sizeof( buf ) );
+        strlcat_utf8( buf, name->val.s.s, sizeof(buf), TR_PATH_DELIMITER );
+        tr_free( inf->files[0].name );
+        inf->files[0].name = tr_strdup( buf );
 
         inf->files[0].length = length->val.i;
         inf->totalSize      += length->val.i;
     }
     else
     {
-        tr_err( "%s \"files\" entry and %s \"length\" entry",
-                ( files ? "Invalid" : "Missing" ),
-                ( length ? "invalid" : "missing" ) );
+        tr_err( _( "Invalid or missing metadata entries \"length\" and \"files\"" ) );
     }
 
     return TR_OK;
