@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: utils.c 5010 2008-02-13 01:21:23Z charles $
+ * $Id: utils.c 5300 2008-03-19 02:16:07Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,11 +23,12 @@
  *****************************************************************************/
 
 #include <assert.h>
+#include <ctype.h> /* isalpha */
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <string.h> /* strerror */
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -59,6 +60,33 @@ void tr_msgInit( void )
          messageLock = tr_lockNew( );
 }
 
+/*
+#if defined(SYS_DARWIN)
+
+const char * tr_getMacLocalizedCString(CFStringRef string)
+{
+    //if you need to be thread safe, add proper locks around that
+    static CFMutableDictionaryRef mapping = NULL;
+    if( mapping == NULL )
+    {
+        mapping = CFDictionaryCreateMutable(NULL, 0, &kCFTypeDictionaryKeyCallBacks, NULL);
+    }
+    
+    const char * result = (const char *)CFDictionaryGetValue(mapping, string);
+    if( result == NULL )
+    {
+        CFStringRef localizedString = CFCopyLocalizedStringFromTable(string, CFSTR("LibraryLocalizable"), NULL);
+        result = (const char *)malloc(CFStringGetMaximumSizeForEncoding(CFStringGetLength(localizedString), kCFStringEncodingUTF8));
+        CFStringGetCString(localizedString, (char *)result, CFStringGetLength(localizedString)+1, kCFStringEncodingUTF8);
+        CFDictionarySetValue(mapping, string, result);
+    }
+    
+    return result;
+}
+
+#endif
+*/
+
 FILE*
 tr_getLog( void )
 {
@@ -115,7 +143,8 @@ tr_setMessageLevel( int level )
     tr_lockUnlock( messageLock );
 }
 
-int tr_getMessageLevel( void )
+int
+tr_getMessageLevel( void )
 {
     int ret;
 
@@ -127,7 +156,8 @@ int tr_getMessageLevel( void )
     return ret;
 }
 
-void tr_setMessageQueuing( int enabled )
+void
+tr_setMessageQueuing( int enabled )
 {
     tr_msgInit();
     tr_lockLock( messageLock );
@@ -135,7 +165,8 @@ void tr_setMessageQueuing( int enabled )
     tr_lockUnlock( messageLock );
 }
 
-tr_msg_list * tr_getQueuedMessages( void )
+tr_msg_list *
+tr_getQueuedMessages( void )
 {
     tr_msg_list * ret;
 
@@ -149,7 +180,8 @@ tr_msg_list * tr_getQueuedMessages( void )
     return ret;
 }
 
-void tr_freeMessageList( tr_msg_list * list )
+void
+tr_freeMessageList( tr_msg_list * list )
 {
     tr_msg_list * next;
 
@@ -157,6 +189,7 @@ void tr_freeMessageList( tr_msg_list * list )
     {
         next = list->next;
         free( list->message );
+        free( list->name );
         free( list );
         list = next;
     }
@@ -191,12 +224,14 @@ tr_asprintf( char **strp, const char *fmt, ...)
 }
 
 void
-tr_msg( const char * file, int line, int level, const char * fmt, ... )
+tr_msg( const char * file, int line, int level,
+        const char * name,
+        const char * fmt, ... )
 {
     FILE * fp;
 
-    assert( NULL != messageLock );
-    tr_lockLock( messageLock );
+    if( messageLock )
+        tr_lockLock( messageLock );
 
     fp = tr_getLog( );
 
@@ -228,6 +263,7 @@ tr_msg( const char * file, int line, int level, const char * fmt, ... )
                 newmsg->message = text;
                 newmsg->file = file;
                 newmsg->line = line;
+                newmsg->name = tr_strdup( name );
 
                 *messageQueueTail = newmsg;
                 messageQueueTail = &newmsg->next;
@@ -243,7 +279,8 @@ tr_msg( const char * file, int line, int level, const char * fmt, ... )
         }
     }
 
-    tr_lockUnlock( messageLock );
+    if( messageLock )
+        tr_lockUnlock( messageLock );
 }
 
 int tr_rand( int sup )
@@ -360,13 +397,13 @@ tr_loadFile( const char * path, size_t * size )
     errno = 0;
     if( stat( path, &sb ) )
     {
-        tr_err( "Couldn't get information for file \"%s\" %s", path, strerror(errno) );
+        tr_err( _( "Couldn't read file \"%s\": %s" ), path, tr_strerror(errno) );
         return NULL;
     }
 
     if( ( sb.st_mode & S_IFMT ) != S_IFREG )
     {
-        tr_err( "Not a regular file (%s)", path );
+        tr_err( _( "Couldn't read file \"%s\": %s" ), path, _( "Not a regular file" ) );
         return NULL;
     }
 
@@ -374,20 +411,20 @@ tr_loadFile( const char * path, size_t * size )
     file = fopen( path, "rb" );
     if( !file )
     {
-        tr_err( "Couldn't open file \"%s\" %s", path, strerror(errno) );
+        tr_err( _( "Couldn't read file \"%s\": %s" ), path, tr_strerror(errno) );
         return NULL;
     }
     buf = malloc( sb.st_size );
     if( NULL == buf )
     {
-        tr_err( "Couldn't allocate memory (%"PRIu64" bytes)",
-                ( uint64_t )sb.st_size );
+        tr_err( _( "Couldn't read file \"%s\": %s" ), path, _( "Memory allocation failed" ) );
         fclose( file );
+        return NULL;
     }
     fseek( file, 0, SEEK_SET );
     if( fread( buf, sb.st_size, 1, file ) != 1 )
     {
-        tr_err( "Error reading \"%s\" %s", path, strerror(errno) );
+        tr_err( _( "Couldn't read file \"%s\": %s" ), path, tr_strerror(errno) );
         free( buf );
         fclose( file );
         return NULL;
@@ -407,6 +444,8 @@ tr_mkdir( const char * path, int permissions
                                                     )
 {
 #ifdef WIN32
+    if( path && isalpha(path[0]) && path[1]==':' && !path[2] )
+        return 0;
     return mkdir( path );
 #else
     return mkdir( path, permissions );
@@ -440,7 +479,7 @@ tr_mkdirp( const char * path_in, int permissions )
             /* Folder doesn't exist yet */
             if( tr_mkdir( path, permissions ) ) {
                 const int err = errno;
-                tr_err( "Couldn't create directory %s (%s)", path, strerror( err ) );
+                tr_err( _( "Couldn't create \"%s\": %s" ), path, tr_strerror( err ) );
                 tr_free( path );
                 errno = err;
                 return -1;
@@ -449,7 +488,9 @@ tr_mkdirp( const char * path_in, int permissions )
         else if( ( sb.st_mode & S_IFMT ) != S_IFDIR )
         {
             /* Node exists but isn't a folder */
-            tr_err( "Remove %s, it's in the way.", path );
+            char buf[MAX_PATH_LENGTH];
+            snprintf( buf, sizeof( buf ), _( "File \"%s\" is in the way" ), path );
+            tr_err( _( "Couldn't create \"%s\": %s" ), path_in, buf );
             tr_free( path );
             errno = ENOTDIR;
             return -1;
@@ -488,9 +529,9 @@ tr_buildPath ( char *buf, size_t buflen, const char *first_element, ... )
 }
 
 int
-tr_ioErrorFromErrno( void )
+tr_ioErrorFromErrno( int err )
 {
-    switch( errno )
+    switch( err )
     {
         case 0:
             return TR_OK;
@@ -504,7 +545,7 @@ tr_ioErrorFromErrno( void )
         case EFBIG:
             return TR_ERROR_IO_FILE_TOO_BIG;
         default:
-            tr_dbg( "generic i/o errno from errno: %s", strerror( errno ) );
+            tr_dbg( "generic i/o errno from errno: %s", tr_strerror( errno ) );
             return TR_ERROR_IO_OTHER;
     }
 }
@@ -515,28 +556,40 @@ tr_errorString( int code )
     switch( code )
     {
         case TR_OK:
-            return "No error";
+            return _( "No error" );
 
         case TR_ERROR:
-            return "Generic error";
+            return _( "Unspecified error" );
         case TR_ERROR_ASSERT:
-            return "Assert error";
+            return _( "Assert error" );
 
+        case TR_ERROR_IO_PARENT:
+            return _( "Destination folder doesn't exist" );
         case TR_ERROR_IO_PERMISSIONS:
-            return "Insufficient permissions";
+            return tr_strerror( EACCES );
         case TR_ERROR_IO_SPACE:
-            return "Insufficient free space";
+            return tr_strerror( ENOSPC );
         case TR_ERROR_IO_FILE_TOO_BIG:
-            return "File too large";
+            return tr_strerror( EFBIG );
         case TR_ERROR_IO_OPEN_FILES:
-            return "Too many open files";
+            return tr_strerror( EMFILE );
         case TR_ERROR_IO_DUP_DOWNLOAD:
-            return "Already active transfer with same name and download folder";
+            return _( "A torrent with this name and destination folder already exists." );
+        case TR_ERROR_IO_CHECKSUM:
+            return _( "Checksum failed" );
         case TR_ERROR_IO_OTHER:
-            return "Generic I/O error";
+            return _( "Unspecified I/O error" );
+
+        case TR_ERROR_TC_ERROR:
+            return _( "Tracker error" );
+        case TR_ERROR_TC_WARNING:
+            return _( "Tracker warning" );
+
+        case TR_ERROR_PEER_MESSAGE:
+            return _( "Peer sent a bad message" );
 
         default:
-            return "Unknown error";
+            return _( "Unknown error" );
     }
 }
 
@@ -595,6 +648,15 @@ tr_free( void * p )
         free( p );
 }
 
+const char*
+tr_strerror( int i )
+{
+    const char * ret = strerror( i );
+    if( ret == NULL )
+        ret = "Unknown Error";
+    return ret;
+}
+
 /****
 *****
 ****/
@@ -658,49 +720,72 @@ int
 tr_bitfieldHas( const tr_bitfield * bitfield, size_t nth )
 {
     static const uint8_t ands[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-    return bitfield!=NULL && (bitfield->bits[nth>>3u] & ands[nth&7u] );
+    const size_t i = nth >> 3u;
+    return ( bitfield != NULL )
+        && ( bitfield->bits != NULL )
+        && ( i < bitfield->len )
+        && ( ( bitfield->bits[i] & ands[nth&7u] ) != 0 );
 }
 
-void
+int
 tr_bitfieldAdd( tr_bitfield  * bitfield, size_t nth )
 {
     static const uint8_t ands[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-    bitfield->bits[nth>>3u] |= ands[nth&7u];
+    const size_t i = nth >> 3u;
+
+    assert( bitfield != NULL );
+    assert( bitfield->bits != NULL );
+
+    if( i >= bitfield->len )
+        return -1;
+
+    bitfield->bits[i] |= ands[nth&7u];
     assert( tr_bitfieldHas( bitfield, nth ) );
+    return 0;
 }
 
-void
+int
 tr_bitfieldAddRange( tr_bitfield  * bitfield,
                      size_t         begin,
                      size_t         end )
 {
-    /* TODO: there are faster ways to do this */
+    int err = 0;
     size_t i;
     for( i=begin; i<end; ++i )
-        tr_bitfieldAdd( bitfield, i );
+        if(( err = tr_bitfieldAdd( bitfield, i )))
+            break;
+    return err;
 }
 
-void
+int
 tr_bitfieldRem( tr_bitfield   * bitfield,
                 size_t          nth )
 {
     static const uint8_t rems[8] = { 127, 191, 223, 239, 247, 251, 253, 254 };
+    const size_t i = nth >> 3u;
 
-    if( bitfield != NULL )
-        bitfield->bits[nth>>3u] &= rems[nth&7u];
+    assert( bitfield != NULL );
+    assert( bitfield->bits != NULL );
 
+    if( i >= bitfield->len )
+        return -1;
+
+    bitfield->bits[i] &= rems[nth&7u];
     assert( !tr_bitfieldHas( bitfield, nth ) );
+    return 0;
 }
 
-void
+int
 tr_bitfieldRemRange ( tr_bitfield  * b,
                       size_t         begin,
                       size_t         end )
 {
-    /* TODO: there are faster ways to do this */
+    int err = 0;
     size_t i;
     for( i=begin; i<end; ++i )
-        tr_bitfieldRem( b, i );
+        if(( err = tr_bitfieldRem( b, i )))
+            break;
+    return err;
 }
 
 tr_bitfield*
@@ -793,6 +878,9 @@ strlcpy(char *dst, const char *src, size_t siz)
 	const char *s = src;
 	size_t n = siz;
 
+	assert( s != NULL );
+	assert( d != NULL );
+
 	/* Copy as many bytes as will fit */
 	if (n != 0) {
 		while (--n != 0) {
@@ -872,3 +960,16 @@ tr_getRatio( double numerator, double denominator )
 
     return ratio;
 }
+
+void
+tr_sha1_to_hex( char * out, const uint8_t * sha1 )
+{
+    static const char hex[] = "0123456789abcdef";
+    int i;
+    for (i = 0; i < 20; i++) {
+        unsigned int val = *sha1++;
+        *out++ = hex[val >> 4];
+        *out++ = hex[val & 0xf];
+    }
+    *out = '\0';
+}
